
% Module for element mass matrix for reactive term
%
% Output: Mtimeelem  ====== 3 x 3 matrix
%
% BMelem(1,1), BMelem(1,2), BMelem(2,1), BMelem(2.2) to be computed in this routine.
%
% elmatbnd(i,1), elmatbnd(i,2) give the index numbers of the vertices of boundary element i
%
% x(elmatbnd(i,j)), y(elmatbnd(i,j)) give the coordinates of the vertices
%
% i = index number of element, imported from AssemblyStepBoundaryMassMatrix.m
%
% BMelem(index1,index2) = (phi(elmatbnd(i,index1)),phi(elmatbnd(i,index2))
%

clear xc
clear yc
BMuuelem = sparse(3,3);
BMvvelem = BMuuelem;
BMppelem = BMuuelem;
BMelem   = sparse(8,8);

for index1=1:topologybnd
	xc(index1) = x(elmatbnd(i,index1));
	yc(index1) = y(elmatbnd(i,index1));
end;

lek = sqrt((xc(3)-xc(1))^2 + (yc(3)-yc(1))^2);

for index1 = 1:topologylinbnd
	if (yc(2*index1-1) == 8)
		K_B(1) = 0;
		K_B(2) = 0;
		if (abs(xc(2*index1-1) - 4) < 0.8)
			tau(1) = tau_orig(1);
			tau(2) = tau_orig(2);
		else
			tau(1) = 0;
			tau(2) = 0;
		end;
	else
		K_B(1) = K_orig(1);
		K_B(2) = K_orig(2);
		tau(1) = tau_orig(1);
		tau(2) = tau_orig(2);
	end;
	BMuuelem(2*index1-1,2*index1-1) = -K_B(1) * lek / 6;
	BMvvelem(2*index1-1,2*index1-1) = -K_B(2) * lek / 6;
end;

if (yc(2) == 8)
	K_B(1) = 0;
	K_B(2) = 0;
	if (abs(xc(2) - 4) < 0.8)
		tau(1) = tau_orig(1);
		tau(2) = tau_orig(2);
	else
		tau(1) = 0;
		tau(2) = 0;
	end;
else
	K_B(1) = K_orig(1);
	K_B(2) = K_orig(2);
	tau(1) = tau_orig(1);
	tau(2) = tau_orig(2);
end;

BMuuelem(2,2) = -K_B(1) * 2 * lek / 3;
BMvvelem(2,2) = -K_B(2) * 2 * lek / 3;

for index1 = 1:topologylinbnd
	if (yc(2*index1-1) == 0) | ((yc(2*index1-1) == 8) & (abs(xc(2*index1-1)-4) < 0.8))
		K_B(3) = 0;
	else
		K_B(3) = K_orig(3);
	end;
	if (xc(2*index1-1) == 0) | (xc(2*index1-1) == 8)
		K_B(3) = K_orig(3);
	end;
	BMppelem(2*index1-1,2*index1-1) = K_B(3) * lek / 2 * dt;
end;

seqarr(1) = 1;
for index1 = 1:topologybnd-1
	ind1 = elmatbnd(i,index1);
	seqarr(index1+1) = seqarr(index1) + unklist(ind1,4);
end;

% seqarr = [1 4 6];

% seqarr contains the indexes of the first unknown (u) at a node

% Creation of total boundary element matrix

for index1 = 1:topologybnd
	for index2 = 1:topologybnd
		BMelem(seqarr(index1),seqarr(index2)) = BMuuelem(index1,index2);
		BMelem(seqarr(index1)+1,seqarr(index2)+1) = BMvvelem(index1,index2);
	end;
end;

for index1 = 1:2
	for index2 = 1:2
		BMelem(seqarr(2*index1-1)+2,seqarr(2*index2-1)+2) = BMppelem(2*index1-1,2*index2-1);
	end;
end;
