
% Module for element mass matrix for reactive term
%
% Output: Selem  ====== 2 x 2 matrix
%
% Selem(1,1), Selem(1,2), Selem(1,3), Selem(2,1), Selem(2.2), Selem(2,3),
%
% Selem(3,1), Selem(3,2), Selem(3,3) to be computed in this routine.
%
% elmat(i,1), elmat(i,2), elmat(i,3) give the index numbers of the vertices of element i
%
% x(elmat(i,j)), y(elmat(i,j)) give the coordinates of the vertices
%
% i = index number of element, imported from AssemblyStepStiffnessMatrix.m
%
% Selem(index1,index2) = (grad phi(elmat(i,index1)),grad phi(i,index2))
%

clear xc
clear yc
Suuelem = sparse(topology,topology);
Suvelem = sparse(topology,topology);
Supelem = sparse(topology,topology);
Svuelem = sparse(topology,topology);
Svvelem = sparse(topology,topology);
Svpelem = sparse(topology,topology);
Spuelem = sparse(topology,topology);
Spvelem = sparse(topology,topology);
Sppelem = sparse(topology,topology);
SLelem  = sparse(15,15);
SRelem  = SLelem;

for index1 = 1:topology
	xc(index1) = x(elmat(i,index1));
	yc(index1) = y(elmat(i,index1));
end;

Delta = xc(3)*yc(5)-xc(5)*yc(3)+xc(5)*yc(1)-xc(1)*yc(5)+xc(1)*yc(3)-xc(3)*yc(1);

b(1) = (yc(3)-yc(5)) / Delta;  % for basis function 1
c(1) = (xc(5)-xc(3)) / Delta;

b(2) = (yc(5)-yc(1)) / Delta;  % for basis function 3
c(2) = (xc(1)-xc(5)) / Delta;

b(3) = (yc(1)-yc(3)) / Delta;  % for basis function 5
c(3) = (xc(3)-xc(1)) / Delta;

lambdamatrix = [0.5   0.5   0;
                0     0.5   0.5;
                0.5   0     0.5];

% lambdamatrix(j,i) = lambda_{2i-1} (x_{2j})

seqarr(1) = 1;
for index1 = 1:topology-1
        ind1 = elmat(i,index1);
	seqarr(index1+1) = seqarr(index1) + unklist(ind1,4);
end;

%seqarr = [1;4;6;9;11;14]; % indexes of first unknown (u) at a node

% seqarr contains the indexes of the first unknown (u) at a node 

% dphidx(i,j) = dphi_i/dx(x_j,y_j)
% dphidy(i,j) = dphi_i/dy(x_j,y_j)

%gradient of odd basis functions phi in integration points at midpoints
for index1 = 1:topology/2 % over odd basis functions at vertices
	for index2 = 1:topology/2 % over integration points, ie midpoints
		dphidx(2*index1-1,index2) = (4*lambdamatrix(index2,index1)-1)*b(index1);
		dphidy(2*index1-1,index2) = (4*lambdamatrix(index2,index1)-1)*c(index1);
	end;
end;

% Now for the even basis functions corresponding to the midpoints
for index2 = 1:topology/2 % over integration points, ie midpoints
	dphidx(2,index2) = 4*(lambdamatrix(index2,1)*b(2) + lambdamatrix(index2,2)*b(1));
	dphidx(4,index2) = 4*(lambdamatrix(index2,2)*b(3) + lambdamatrix(index2,3)*b(2));
	dphidx(6,index2) = 4*(lambdamatrix(index2,1)*b(3) + lambdamatrix(index2,3)*b(1));
	dphidy(2,index2) = 4*(lambdamatrix(index2,1)*c(2) + lambdamatrix(index2,2)*c(1));
	dphidy(4,index2) = 4*(lambdamatrix(index2,2)*c(3) + lambdamatrix(index2,3)*c(2));
	dphidy(6,index2) = 4*(lambdamatrix(index2,1)*c(3) + lambdamatrix(index2,3)*c(1));
end;

%%%%%% Computation of matrix elements

for index1 = 1:topology
	for index2 = 1:topology
		Suuelem(index1,index2) = 0;
		Suvelem(index1,index2) = 0;
		Svuelem(index1,index2) = 0;
		Svvelem(index1,index2) = 0;
		for index3 = 1 : topology/2 % sum over de midpoints of the sides
			Suuelem(index1,index2) = Suuelem(index1,index2) + abs(Delta)/6*((lambda+2*mu)*dphidx(index1,index3)*dphidx(index2,index3) + mu*dphidy(index1,index3)*dphidy(index2,index3));

			Suvelem(index1,index2) = Suvelem(index1,index2) + abs(Delta)/6*(lambda+mu)*dphidx(index1,index3)*dphidy(index2,index3);

			Svuelem(index1,index2) = Svuelem(index1,index2) + abs(Delta)/6*(lambda+mu)*dphidy(index1,index3)*dphidx(index2,index3);

			Svvelem(index1,index2) = Svvelem(index1,index2) + abs(Delta)/6*(mu*dphidx(index1,index3)*dphidx(index2,index3) + (lambda+2*mu)*dphidy(index1,index3)*dphidy(index2,index3));
		end; 
	end;
end;

for index1 = 1:topology/2
	for index2 = 1:topology/2
		Supelem(2*index1,2*index2-1) = abs(Delta)/6*b(index2);
		Svpelem(2*index1,2*index2-1) = abs(Delta)/6*c(index2);
	end; 
end;

for index1 = 1:topology/2
	for index2 = 1:topology
		Spuelem(2*index1-1,index2) = 0;
		Spvelem(2*index1-1,index2) = 0;
		for index3 = 1:topology/2
			Spuelem(2*index1-1,index2) = Spuelem(2*index1-1,index2) + abs(Delta)/6*lambdamatrix(index3,index1)*dphidx(index2,index3);
			Spvelem(2*index1-1,index2) = Spvelem(2*index1-1,index2) + abs(Delta)/6*lambdamatrix(index3,index1)*dphidy(index2,index3);
		end;
	end;
	for index2 = 1:topology/2
		Sppelem(2*index1-1,2*index2-1) = abs(Delta)/2*alpha*(b(index1)*b(index2)+c(index1)*c(index2));
	end;
end;

% creation of total element matrices

for index1 = 1:topology
	for index2 = 1:topology
		SLelem(seqarr(index1),seqarr(index2))     = Suuelem(index1,index2);
		SLelem(seqarr(index1),seqarr(index2)+1)   = Suvelem(index1,index2);
		SLelem(seqarr(index1)+1,seqarr(index2))   = Svuelem(index1,index2);
		SLelem(seqarr(index1)+1,seqarr(index2)+1) = Svvelem(index1,index2);
	end;
	for index2 = 1:topology/2
		SLelem(seqarr(index1),seqarr(2*index2-1)+2) = Supelem(index1,2*index2-1);
		SLelem(seqarr(index1)+1,seqarr(2*index2-1)+2) = Svpelem(index1,2*index2-1);
	end;
end;

for index1 = 1:topology/2
	for index2 = 1:topology
		SLelem(seqarr(2*index1-1)+2,seqarr(index2)) = Spuelem(2*index1-1,index2);

		SRelem(seqarr(2*index1-1)+2,seqarr(index2)) = SLelem(seqarr(2*index1-1)+2,seqarr(index2));

		SLelem(seqarr(2*index1-1)+2,seqarr(index2)+1) = Spvelem(2*index1-1,index2);

		SRelem(seqarr(2*index1-1)+2,seqarr(index2)+1) = SLelem(seqarr(2*index1-1)+2,seqarr(index2)+1);
	end;
% For time integration, a multiplication by dt is necessary
	for index2 = 1:topology/2
		SLelem(seqarr(2*index1-1)+2,seqarr(2*index2-1)+2) = dt*Sppelem(2*index1-1,2*index2-1);
	end;
end;
