
% Matlab code for Finite Elements Solution of 

% du/dt + v grad u = D div grad u + f(x) - lambda u
%
% u(x,0) = u_0(x)
%
% D dc/dn = K_L (c - c_L) at boundary
%
% S is for the diffusion term
%
% M is the mass matrix for the reaction term
%
% Mtime is the mass matrix for the time derivative
%
% C is for the convection term
%
%
% To be done:  	- Several alternative arrangements of the elements (equilateral triangles)
%
%		- Convection has been implemented using SGA, SUPG could be implemented.
%
%		- higher order elements (quadratic - bilinear)
%
%		- extensions to various other problems (Cahn-Hilliard / Mechanics / Level Set)
%
% This routine, FredMesh.m, is the mesh generation routine.
%

clear all

InputParameters;

n 		= nx * ny; % Total number of gridnodes

GenerateUnklist;

ndfd		= unklist(n,3);

SL 		= sparse(ndfd,ndfd);
SR 		= SL;

f 		= zeros(ndfd,1);

u 		= zeros(ndfd,1);

topology	= 6; % quadratic triangles
topologylin	= 3;
topologybnd	= 3;
topologylinbnd	= 2;

GenerateTopology;

GenerateMesh;

if Meshplot == 1
	PlotMesh; % plot of mesh
end;
